/*
	E-Forecast, shows an icon with the current weather forecast.
	Copyright (c) 2000 Roberto Machorro <rmach@bigfoot.com>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <epplet.h>

#define __VERSION		"0.2"

#define __TIME_TIL_REFRESH	"60.0"
#define __WEATHER_SITE		"http://www.weather.com/weather/local/"
#define __WEATHER_CITY_FILE	"MXXX0046"
#define __FORECAST_STRING	"http://image.weather.com/web/common/wxicons/"
#define __UPDATE_IMAGE		EROOT "/epplet_data/E-Forecast/update.png"
#define __TIMER_NAME		"TIMER"

int post_update = 1;
Window config_window = None;
double refresh_time = 60.0;
char *weather_city_file = NULL;
Epplet_gadget icon_image = NULL;
Epplet_gadget b_close, b_help, b_config;
Epplet_gadget cfg_time_refresh, cfg_city_file;

void change_image(char *file_name)
{
	if (icon_image != NULL) {
		Epplet_gadget_destroy(icon_image);
	}
	icon_image = Epplet_create_image(2, 2, 44, 44, file_name);
	Epplet_gadget_show(icon_image);
}

void assign_configuration()
{
	refresh_time = (double)atof(Epplet_query_config_def("refresh_time",
						__TIME_TIL_REFRESH));
	weather_city_file = Epplet_query_config_def("weather_city_file",
						__WEATHER_CITY_FILE);
}

void fetch_forecast_image_url(char *img_url)
{
	FILE *ph;
	char str_cmd[255];
	char str_url[200];
	Esnprintf(str_url, sizeof(str_url), "%s%s", __WEATHER_SITE,
						weather_city_file);
	Esnprintf(str_cmd, sizeof(str_cmd),
	"wget -q -O - %s|grep %s|sed 1d|awk -F'\"' '{print $2}'|head -n 1",
					str_url, __FORECAST_STRING);
	ph = popen(str_cmd, "r");
	if (ph != NULL) {
		if (fgets(img_url, 255, ph) == NULL) {
			strcpy(img_url, "");
			Epplet_dialog_ok("Could not get URL...");
		}
		pclose(ph);
	} else {
		strcpy(img_url, "");
		Epplet_dialog_ok("Could not execute update command!");
	}
}

static void cb_in(void *data, Window window)
{
	if (window == Epplet_get_main_window()) {
		Epplet_gadget_show(b_close);
		Epplet_gadget_show(b_config);
		Epplet_gadget_show(b_help);
	}
	return;
	data = NULL;
}

static void cb_out(void *data, Window window)
{
	if (window == Epplet_get_main_window()) {
		Epplet_gadget_hide(b_close);
		Epplet_gadget_hide(b_config);
		Epplet_gadget_hide(b_help);
	}
	return;
	data = NULL;
}

static void cb_close(void *data)
{
	Epplet_unremember();
	Esync();
	exit(0);
	data = NULL;
}

static void cb_help(void *data)
{
	Epplet_show_about("E-Forecast");
	data = NULL;
}

static void cb_timer(void *data)
{
	char *tmp_name;
	char img_url[255];
	char img_cmd[300];
	if (post_update) {
		post_update = 0;
		fetch_forecast_image_url(img_url);
		if (strlen(img_url) < 0) {
			return;
		}
		tmp_name = tempnam("/tmp", ".E-F-");
		Esnprintf(img_cmd, sizeof(img_cmd),
				"wget -q -O %s %s", tmp_name, img_url);
		Epplet_run_command(img_cmd);
		change_image(tmp_name);
		unlink(tmp_name);
		free(tmp_name);
		Epplet_timer(cb_timer, NULL, refresh_time*60.0, __TIMER_NAME);
	} else {
		post_update = 1;
		change_image(__UPDATE_IMAGE);
		Epplet_timer(cb_timer, NULL, 0.5, __TIMER_NAME);
	}
	data = NULL;
}

static void cb_ok_cfg(void *data)
{
	char buffer[255];
	Epplet_window_destroy(config_window);
	config_window = None;
	Epplet_remove_timer(__TIMER_NAME);
	strcpy(buffer, Epplet_textbox_contents(cfg_time_refresh));
	Epplet_modify_config("refresh_time", buffer);
	strcpy(buffer, Epplet_textbox_contents(cfg_city_file));
	Epplet_modify_config("weather_city_file", buffer);
	assign_configuration();
	post_update = 1;
	change_image(__UPDATE_IMAGE);
	Epplet_timer(cb_timer, NULL, 0.5, __TIMER_NAME);
	Epplet_save_config();
	return;
	data = NULL;
}

static void cb_cancel_cfg(void *data)
{
	Epplet_window_destroy(config_window);
	config_window = None;
	return;
	data = NULL;
}

static void cb_config(void *data)
{
	char buffer[128];
	if (config_window) {
		Epplet_dialog_ok("A configuration window is already open.");
		return;
	}

	config_window = Epplet_create_window_config(245, 80,
					"E-Forecast Configuration",
					cb_ok_cfg, NULL,
					NULL, NULL,
					cb_cancel_cfg, NULL);

	Epplet_gadget_show(Epplet_create_label(4, 5, "Refresh (mins):", 2));
	Esnprintf(buffer, sizeof(buffer), "%3.2f", refresh_time);
	cfg_time_refresh = Epplet_create_textbox(NULL, buffer, 100, 4,
						140, 20, 2, NULL, NULL);
	Epplet_gadget_show(cfg_time_refresh);

	Epplet_gadget_show(Epplet_create_label(4, 28, "City HTML file:", 2));
	cfg_city_file = Epplet_create_textbox(NULL, weather_city_file, 100, 27,
						140, 20, 2, NULL, NULL);
	Epplet_gadget_show(cfg_city_file);

	Epplet_window_show(config_window);
	Epplet_window_pop_context();
	return;
	data = NULL;
}

int main(int argc, char **argv)
{
	atexit(Epplet_cleanup);

	Epplet_Init("E-Forecast", "0.1", "Enlightened Weather Forecast",
			3, 3, argc, argv, 0);
	Epplet_load_config();
	assign_configuration();

	Epplet_timer(cb_timer, NULL, 0.5, __TIMER_NAME);

	b_close = Epplet_create_button(NULL, NULL, 2, 2, 0, 0, "CLOSE",
			0, NULL, cb_close, NULL);
	b_config = Epplet_create_button(NULL, NULL, 34, 34, 0, 0, "CONFIGURE",
			0, NULL, cb_config, NULL);
	b_help = Epplet_create_button(NULL, NULL, 34, 2, 0, 0, "HELP",
			0, NULL, cb_help, NULL);

	change_image(__UPDATE_IMAGE);

	Epplet_register_focus_in_handler(cb_in, NULL);
	Epplet_register_focus_out_handler(cb_out, NULL);

	Epplet_show();
	Epplet_Loop();

	return 0;
}
